% P39Estimator2.m Estimators 2
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2013. All rights reserved.
 
%% init vars
post_title = 'P39Estimator2';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
MakeBlogPostFileLists(post_title);

%% set matlab path
addpath('L:\Projects\CountDetPaper\Code');
addpath('L:\Projects\DEbook\SolveEqns');
addpath('L:\Projects\DEsolvePaper\Code');
addpath('L:\Projects\CountDetPaper\Revised\Code');
addpath('L:\Projects\DEsolvePaper\Revisions-Reviews-1\MakeFiguresCode');
addpath('L:\Projects\DEsolvePaper\Revisions-Reviews-2\MakeFiguresCode');
addpath('L:\Projects\DEsolvePaper\PoissonSims');
addpath('L:\Projects\Blog\Posts\P38Estimator1\code');
linespecs = {'-k',':b','--r','-.k*'};

%% show errors outside calibration region with least squares fit 
   % polynomial fit to data in calibration region
x =  linspace(-1,2)';
y = zeros(size(x));
y((x>0) & (x<1)) = 1;
norder = 9;
p = polyfit(x,y,norder);
fh = figure;
hold on
plot(x,y,'-k','linewidth',2);
    % evaluate outside the region
xbig = linspace(-2,3);
plot(xbig,polyval(p,xbig),'-r','linewidth',1.5);
hold off
axis([-2 3 0 5]) % show fit in whole region
hl = legend('data to fit',sprintf('%d order fit',norder),'location','north');
set(gca,'fontsize',14','linewidth',1)
psn = [125 250 675 500];
set(fh,'color','w','Position',psn)

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'Polyfit2Step';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% Create the spectrum, mu data
tube_voltage = 120; % keV  
[specnum,xray_energies,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
nenergies = numel(xray_energies);
   % mu data
materials = {'Al','C5O2H8'};
rhos = [2.699 1.19];
zmus =  rhos(1)*xraymu(materials{1},xray_energies) + 1i*rhos(2)*xraymu(materials{2},xray_energies);
   % create the spectrum data (specdat) struct
specdat.specnum = specdat.specnum_norm; % normalize so can adjust Ninit easily later
specdat.zmus = zmus(:);
specdat.mus = zcomp(zmus(:));
    % compute thresholds to equalize transmitted number of photons for each bin
    % attenuate with median thicknesses in calibration phantom: 2.5 cm Al and 15 cm plastic
sp = specdat;
sp.specnum = specdat.specnum.*exp(-2.5*real(specdat.zmus)-15*imag(specdat.zmus));
specdat.idx_threshold2 = OptimNbinsThresholds(sp,2);
specdat.idx_threshold3 = OptimNbinsThresholds(sp,3);
clear sp
    
%% Compute calibration and random data for all nbins cases
nsteps = 40;
   % step wedge thicknesses, gm/cm^2
fprintf('\n\n nsteps %d\n',nsteps);
A_calib_region = [5 30];   % boundaries of calibration region
zts = zmeshgrid([0,A_calib_region(1),nsteps],[0,A_calib_region(2),nsteps]);
zts = zts(:);
cdat.As = zcomp(zts);

%% create random data for the simulation 
nrand_trials = 1000; % number of random trials per point
spec_noise = specdat; % first copy all members of structure
Ninit = 4e4;
spec_noise.specnum = Ninit*spec_noise.specnum_norm;
A2plot = 0.5*A_calib_region;
   % closest calib point to test point
[mindst,cdat.idxn] = min(normrow(bsxfun(@minus,cdat.As,A2plot))); % cdat.As defined in previous cell
tdat.As = repmat(A2plot,nrand_trials,1); % t(est)data

dats = struct([]);% vector of structs for each nbin
for kbin = [2,3]
    dat.nbins = kbin;
    spec_noise.idx_threshold = specdat.(['idx_threshold' num2str(kbin)]);
    dat.spec_noise = spec_noise;
        % use zero noise data for calibration
    cdat.ns = PHAbinCounts(cdat.As,spec_noise,spec_noise.idx_threshold);
      % the table has 0,0 as the first row
       % divide by 0 thickness value to minimize  constant term
    cdat.logI = -log(bsxfun(@rdivide,cdat.ns,cdat.ns(1,:)));  
    cdat.logI_mn = mean(cdat.logI);

    lambdas = PHAbinCounts( tdat.As ,spec_noise, spec_noise.idx_threshold);
    tdat.ns = poissrnd(lambdas);
    assert( ~any(tdat.ns(:) == 0) );
    tdat.logI = -log(bsxfun(@rdivide,tdat.ns,cdat.ns(1,:))); % 1st row of cdat.ns is for 0 thickness
    tdat.logI_mn = mean(tdat.logI);
    
    dat.cdat = cdat;
    dat.tdat = tdat;
    dats = AddStruct2StructArray(dats,dat);
end
%% estimates for data on line through mean for nbin = 2
idx2 = find([dats.nbins]==2);
cdat = dats(idx2).cdat;
tdat = dats(idx2).tdat;
spec_noise = dats(idx2).spec_noise;
n2plot = 50;
rangets = max(tdat.logI(:,1)) - min(tdat.logI(:,1));
ts2plot = tdat.logI_mn(1) + (linspace(0,rangets,n2plot) - 0.5*rangets);
tdat_line.logI = [ts2plot(:) tdat.logI_mn(:,2)*ones(n2plot,1)];
tdat_line.ns = exp(bsxfun(@minus, log(cdat.ns(1,:)),tdat_line.logI));
tdat_line.As = repmat(A2plot,n2plot,1);
dat_mle = IterMLEsolve(spec_noise,tdat_line);
dat_poly = PolyASolveEquations(cdat,tdat_line);   

%% plot the random data vs calibration region for nbins == 2
fh = figure;
set(fh,'color','w','position',[264         468         506        400]);
linewid = 1.5;
hold on
plot(cdat.logI(1:5:end,1),cdat.logI(1:5:end,2),'+k','markersize',3);
plot(tdat.logI(:,1),tdat.logI(:,2),'.r','markersize',1);
plot(tdat_line.logI(:,1),tdat_line.logI(:,2),'-b','linewidth',linewid);
hold off
set(gca,'linewidth',linewid,'fontsize',14);
xlabel('L_1','fontsize',12); ylabel('L_2','fontsize',12); 

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = '2DCalibDataWithNoise';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk','renderer','painters');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot computed A vector components for nbin = 2
fh = figure;
subplot(121)
plot(ts2plot,dat_mle.Astest(:,1),'linewidth',1.5)
hold on
plot(ts2plot,dat_poly.Astest(:,1),'-r','linewidth',1.5)
plot([mnts mxts],[2.5 2.5],'--g') % actual value
hold off
% axis([-2 2 0 10])
xlabel('logI_1','fontsize',14);
ylabel('A_1','fontsize',14);
set(gca,'fontsize',14','linewidth',1.5)
hl = legend('MLE','poly','actual');
subplot(122)
plot(ts2plot,dat_mle.Astest(:,2),'linewidth',1.5)
hold on
plot(ts2plot,dat_poly.Astest(:,2),'-r','linewidth',1.5)
plot([mnts mxts],[15 15],'--g','linewidth',1.5) % actual value
hold off
% axis([-2 2 0 16])
xlabel('logI_1','fontsize',14);
ylabel('A_2','fontsize',14);
set(gca,'fontsize',14','linewidth',1.5)
set(fh,'color','w','position',[200   400   800   400])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'Nbins2Estimates';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk','renderer','painters');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi


%% plot random data vs calibration region for nbins == 3
idx3 = find([dats.nbins]==3);
cdat = dats(idx3).cdat;
tdat = dats(idx3).tdat;

fh = figure;
set(fh,'position',[264         468         506        1030]);
linewid = 1.5;


subplot(121)
   % show calibration data surface--view #1
X = reshape(cdat.logI(:,1),nsteps,nsteps);
Y = reshape(cdat.logI(:,2),nsteps,nsteps);
Z = reshape(cdat.logI(:,3),nsteps,nsteps);
colormap(summer)
hs1 = surf(X,Y,Z);
set(hs1,'EdgeAlpha',0);
hold on,plot3(tdat.logI(:,1),tdat.logI(:,2),tdat.logI(:,3),'.r'),hold off
hax1 = gca;
xlabel('L_1','fontsize',12); ylabel('L_2','fontsize',12); zlabel('L_3','fontsize',12);
set(hax1,'CameraPosition',[-16.9729  -72.5645   39.8936],'Zgrid','on','Xgrid','on','Ygrid','on','linewidth',linewid,'fontsize',12);
% text(-19,-10,-8,'(a)','fontsize',14)

subplot(122)
   % show calibration data surface--view #2
hs1 = surf(X,Y,Z);
set(hs1,'EdgeAlpha',0);
hold on,plot3(tdat.logI(:,1),tdat.logI(:,2),tdat.logI(:,3),'.r'),hold off
hax2 = gca;
xlabel('L_1','fontsize',12); ylabel('L_2','fontsize',12); zlabel('L_3','fontsize',12);
set(hax2,'CameraPosition',[15.6876  -51.8055  -60.1409],'Zgrid','on','Xgrid','on','Ygrid','on','linewidth',linewid,'fontsize',12);
    % insert normal to surface at logI for A2plot
[Nx,Ny,Nz] = surfnorm(X,Y,Z);
normvecs = [Nx(:),Ny(:),Nz(:)]; % one normal vector per row
norm_at_point = normvecs(cdat.idxn,:)/norm(normvecs(cdat.idxn,:)); % select calib point closest to A2plot and normalize to length is one
   % distances of random data from surface: vectorize ts = (tdat.logI - tdat.logI_mn) dot (norm_at_point)
   % assume mean value is at surface i.e 0 bias
v = bsxfun(@minus,tdat.logI, tdat.logI_mn);
ts = sum(bsxfun(@times,v,norm_at_point),2);
ts2plot = linspace(4*min(ts),4*max(ts))';
points_on_perp_line = bsxfun(@plus,bsxfun(@times,repmat(ts2plot,1,3),norm_at_point),tdat.logI_mn);
hold on,plot3(points_on_perp_line(:,1),points_on_perp_line(:,2),points_on_perp_line(:,3),'-k','linewidth',1.5),hold off   

set(fh,'color','w','position',[200   400   800   400])
ht1 = annotation(fh,'textbox','String',{'(a)'},'FitBoxToText','off',...
    'fontsize',13,'Position',[0.031 0.4 0.034 0.08],'EdgeColor',[1 1 1]);

ht2 = annotation(fh,'textbox','String',{'(b)'},'FitBoxToText','off',...
     'fontsize',13,'Position',[0.5 0.4 0.039 0.07],'EdgeColor',[1 1 1]);

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'Nbins3CalibDataWithNoise';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk','renderer','painters');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% prep data for least squares and MLE results on line normal to calibration surface
tdat_line.logI = points_on_perp_line;
tdat_line.ns = exp(bsxfun(@minus, log(cdat.ns(1,:)),tdat_line.logI));
tdat_line.As = repmat(A2plot,100,1);
dat_mle = IterMLEsolve(spec_noise,tdat_line);
dat_poly = PolyASolveEquations(cdat,tdat_line);   

%% plot computed A vector components
fh = figure;
subplot(121)
plot(ts2plot,dat_mle.Astest(:,1),'linewidth',1.5)
hold on
plot(ts2plot,dat_poly.Astest(:,1),'-r','linewidth',1.5)
plot([-2 2],[2.5 2.5],'--g') % actual value
hold off
axis([-2 2 0 10])
xlabel('offset from calib. surface','fontsize',14);
ylabel('A_1','fontsize',14);
set(gca,'fontsize',14','linewidth',1.5)
hl = legend('MLE','poly','actual');
subplot(122)
plot(ts2plot,dat_mle.Astest(:,2),'linewidth',1.5)
hold on
plot(ts2plot,dat_poly.Astest(:,2),'-r','linewidth',1.5)
plot([-2 2],[15 15],'--g','linewidth',1.5) % actual value
hold off
axis([-2 2 0 16])
xlabel('offset from calib. surface','fontsize',14);
ylabel('A_2','fontsize',14);
set(gca,'fontsize',14','linewidth',1.5)
set(fh,'color','w','position',[200   400   800   400])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'MLEvsPolyOnPerpLine';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi
