function z=combinations(n,k)
% combinations: generate all combinations of n objects taken k at a time
% usage: z=combinations(n,k)
%
% arguments:
%     n,k - scalars - n objects taken k at a time
%           for example, n=4, k=2 returns the array
%                 1 1 0 0
%                 1 0 1 0
%                 1 0 0 1
%                 0 1 1 0
%                 0 1 0 1
%                 0 0 1 1
%
%  z - a (C(n,k)xn) matrix with each row a possible outcome
%

% subordinate functions: choose
%
% author: John R. D'Errico
% email: derr...@kodak.com

n=floor(n);
k=floor(k);
if (n<k)|(n<=0)
  z=[];
end

if n==1
  z=(k==1);
elseif k==0
  z=zeros(1,n);
elseif n==k
  z=ones(1,n);
elseif n>k
  % choose(n,m) computes a binomial coefficient = n!/(m!(n-m)!)
  n1=choose(n-1,k-1);
  n0=choose(n-1,k);

z=[[ones(n1,1),combinations(n-1,k-1)];[zeros(n0,1),combinations(n-1,k)]];
end

function ncm=choose(n,m)
% choose: number of combinations of n taken m at a time (binomial
%         coefficient)
% usage: ncm=choose(n,m)
%
% arguments:
%  n,m - scalars - n objects taken m at a time, no replacements
%  
%  ncm - C(n,m)

if n<m
  error('must have n>=m')
end

if (n==m)|(m==0)
  ncm=1;
else
  z=min(m,n-m);
  ncm=prod((n-z+1):n)/prod(1:z);
end 